use leptos::*;
use leptos_router::*;
use serde::{Deserialize, Serialize};

use crate::i18n::*;
use crate::services::contentful::get_rich_text_page;
use crate::services::rich_text;

#[derive(Clone, Debug, Serialize, Deserialize)]
pub struct BlogPostMetadata {
    title: String,
    page_slug: String,
}

#[derive(Params, PartialEq)]
struct BlogPostParams {
    slug: String
}

// #[server(GetBlogPostList, "/api", "GetJson")]
// pub async fn get_blog_post_list() -> Result<Vec<BlogPostMetadata>, ServerFnError> {
//     use crate::services::contentful;
//     let contentful_client = contentful::get_contentful_client();
//     let pages = contentful_client.get_entries_by_type::<contentful::RichTextPageLanguagesWrapper>("richTextPageLanguagesWrapper", None).await;
//     match pages {
//         Ok(found_pages) => {
//             Ok(found_pages.into_iter().map(|page| BlogPostMetadata { title: page.english.title, page_slug: page.english.page_slug }).collect())
//         },
//         Err(e) => Err(ServerFnError::ServerError(e.to_string())),
//     }
// }

#[component]
pub fn BlogList() -> impl IntoView {
    let i18n = use_i18n();
    // let blog_pages_local = create_local_resource(|| (), |_| async move { get_blog_post_list().await });
    // let blog_pages = create_resource(|| (), |_| async move { get_blog_post_list().await });

    view! {
        <main class="main-width">
            <h1>{t!(i18n, welcome_blog)}</h1>
            // <Await future=|| get_blog_post_list() let:pages>
            //     <p>"Loaded " {pages.as_ref().unwrap().len()} " pages"</p>
            // </Await>
            // <Suspense fallback=move || view! { <p>"Loading..."</p> }>
            //     {move || blog_pages_local.get()
            //         .map(|pages| view! { <p>"Loaded " {pages.unwrap().len()} " pages"</p> })
            //     }
            // </Suspense>


            // <Suspense fallback=move || view! { <p>"Loading..."</p> }>
            //     {match blog_pages.get().unwrap_or(Ok(Vec::new())) {
            //         Ok(pages) => {
            //             view! {
            //                 <ul>
            //                     {pages.into_iter()
            //                         .map(|page| {
            //                             let blog_post_url = format!("/blog/{}", page.page_slug);
            //                             view! { <li><A href=blog_post_url>{page.title}</A></li> }
            //                         })
            //                         .collect::<Vec<_>>()}
            //                 </ul>
            //             }
            //         },
            //         Err(error) => view! { <ul>"Error: "{error.to_string()}</ul> }
            //     }}
            //     {move || blog_pages.get()
            //         .map(|pages| view! { <p>"Found " {pages.unwrap().len()} " pages"</p> })
            //     }
            // </Suspense>
        </main>
    }
}

#[component]
pub fn BlogPost() -> impl IntoView {
    let params = use_params::<BlogPostParams>();
    let slug= move || {
        params.with(|params| params.as_ref()
            .map(|params| params.slug.clone())
            .unwrap())
    };
    let blog_page = create_resource(|| (), move |_| async move { get_rich_text_page(slug()).await });

    view! {
        <main class="main-width">
            <Suspense fallback=move || view! { <div>"Loading..."</div> }>
                {match blog_page.get().unwrap_or(Err(ServerFnError::ServerError("Loading...".to_string()))) {
                    Ok(page) => {
                        view! {
                            <div>
                                <h1>{page.english.title}</h1>
                                {rich_text::document_handler(&page.english.rich_text_content)}
                            </div>
                        }
                    },
                    Err(error) => view! { <div>"Error: "{error.to_string()}</div> }
                }}
            </Suspense>
        </main>
    }
}
